@@ -5,16 +5,14 @@ import weakref
 from bson import DBRef, ObjectId, SON
 import pymongo
 import six
+from six import iteritems
 
 from mongoengine.base.common import UPDATE_OPERATORS
-from mongoengine.base.datastructures import (BaseDict, BaseList,
-                                             EmbeddedDocumentList)
+from mongoengine.base.datastructures import BaseDict, BaseList, EmbeddedDocumentList
 from mongoengine.common import _import_class
-from mongoengine.errors import ValidationError
+from mongoengine.errors import DeprecatedError, ValidationError
 
-
-__all__ = ('BaseField', 'ComplexBaseField', 'ObjectIdField',
-           'GeoJsonBaseField')
+__all__ = ("BaseField", "ComplexBaseField", "ObjectIdField", "GeoJsonBaseField")
 
 
 class BaseField(object):
@@ -23,6 +21,7 @@ class BaseField(object):
 
     .. versionchanged:: 0.5 - added verbose and help text
     """
+
     name = None
     _geo_index = False
     _auto_gen = False  # Call `generate` to generate a value
@@ -34,10 +33,21 @@ class BaseField(object):
     creation_counter = 0
     auto_creation_counter = -1
 
-    def __init__(self, db_field=None, name=None, required=False, default=None,
-                 unique=False, unique_with=None, primary_key=False,
-                 validation=None, choices=None, null=False, sparse=False,
-                 **kwargs):
+    def __init__(
+        self,
+        db_field=None,
+        name=None,
+        required=False,
+        default=None,
+        unique=False,
+        unique_with=None,
+        primary_key=False,
+        validation=None,
+        choices=None,
+        null=False,
+        sparse=False,
+        **kwargs
+    ):
         """
         :param db_field: The database field to store this field in
             (defaults to the name of the field)
@@ -52,10 +62,10 @@ class BaseField(object):
             unique with.
         :param primary_key: Mark this field as the primary key. Defaults to False.
         :param validation: (optional) A callable to validate the value of the
-            field.  Generally this is deprecated in favour of the
-            `FIELD.validate` method
+            field.  The callable takes the value as parameter and should raise
+            a ValidationError if validation fails
         :param choices: (optional) The valid choices
-        :param null: (optional) Is the field value can be null. If no and there is a default value
+        :param null: (optional) If the field value can be null. If no and there is a default value
             then the default value is set
         :param sparse: (optional) `sparse=True` combined with `unique=True` and `required=False`
             means that uniqueness won't be enforced for `None` values
@@ -65,7 +75,7 @@ class BaseField(object):
             existing attributes. Common metadata includes `verbose_name` and
             `help_text`.
         """
-        self.db_field = (db_field or name) if not primary_key else '_id'
+        self.db_field = (db_field or name) if not primary_key else "_id"
 
         if name:
             msg = 'Field\'s "name" attribute deprecated in favour of "db_field"'
@@ -81,11 +91,17 @@ class BaseField(object):
         self.sparse = sparse
         self._owner_document = None
 
-        # Validate the db_field
+        # Make sure db_field is a string (if it's explicitly defined).
+        if self.db_field is not None and not isinstance(
+            self.db_field, six.string_types
+        ):
+            raise TypeError("db_field should be a string.")
+
+        # Make sure db_field doesn't contain any forbidden characters.
         if isinstance(self.db_field, six.string_types) and (
-            '.' in self.db_field or
-            '\0' in self.db_field or
-            self.db_field.startswith('$')
+            "." in self.db_field
+            or "\0" in self.db_field
+            or self.db_field.startswith("$")
         ):
             raise ValueError(
                 'field names cannot contain dots (".") or null characters '
@@ -95,15 +111,17 @@ class BaseField(object):
         # Detect and report conflicts between metadata and base properties.
         conflicts = set(dir(self)) & set(kwargs)
         if conflicts:
-            raise TypeError('%s already has attribute(s): %s' % (
-                self.__class__.__name__, ', '.join(conflicts)))
+            raise TypeError(
+                "%s already has attribute(s): %s"
+                % (self.__class__.__name__, ", ".join(conflicts))
+            )
 
         # Assign metadata to the instance
         # This efficient method is available because no __slots__ are defined.
         self.__dict__.update(kwargs)
 
         # Adjust the appropriate creation counter, and save our local copy.
-        if self.db_field == '_id':
+        if self.db_field == "_id":
             self.creation_counter = BaseField.auto_creation_counter
             BaseField.auto_creation_counter -= 1
         else:
@@ -121,11 +139,9 @@ class BaseField(object):
         return instance._data.get(self.name)
 
     def __set__(self, instance, value):
-        """Descriptor for assigning a value to a field in a document.
-        """
-
-        # If setting to None and there is a default
-        # Then set the value to the default value
+        """Descriptor for assigning a value to a field in a document."""
+        # If setting to None and there is a default value provided for this
+        # field, then set the value to the default value.
         if value is None:
             if self.null:
                 value = None
@@ -136,24 +152,29 @@ class BaseField(object):
 
         if instance._initialised:
             try:
-                if (self.name not in instance._data or
-                        instance._data[self.name] != value):
+                value_has_changed = (
+                    self.name not in instance._data
+                    or instance._data[self.name] != value
+                )
+                if value_has_changed:
                     instance._mark_as_changed(self.name)
             except Exception:
-                # Values cant be compared eg: naive and tz datetimes
-                # So mark it as changed
+                # Some values can't be compared and throw an error when we
+                # attempt to do so (e.g. tz-naive and tz-aware datetimes).
+                # Mark the field as changed in such cases.
                 instance._mark_as_changed(self.name)
 
-        EmbeddedDocument = _import_class('EmbeddedDocument')
+        EmbeddedDocument = _import_class("EmbeddedDocument")
         if isinstance(value, EmbeddedDocument):
             value._instance = weakref.proxy(instance)
         elif isinstance(value, (list, tuple)):
             for v in value:
                 if isinstance(v, EmbeddedDocument):
                     v._instance = weakref.proxy(instance)
+
         instance._data[self.name] = value
 
-    def error(self, message='', errors=None, field_name=None):
+    def error(self, message="", errors=None, field_name=None):
         """Raise a ValidationError."""
         field_name = field_name if field_name else self.name
         raise ValidationError(message, errors=errors, field_name=field_name)
@@ -170,11 +191,11 @@ class BaseField(object):
         """Helper method to call to_mongo with proper inputs."""
         f_inputs = self.to_mongo.__code__.co_varnames
         ex_vars = {}
-        if 'fields' in f_inputs:
-            ex_vars['fields'] = fields
+        if "fields" in f_inputs:
+            ex_vars["fields"] = fields
 
-        if 'use_db_field' in f_inputs:
-            ex_vars['use_db_field'] = use_db_field
+        if "use_db_field" in f_inputs:
+            ex_vars["use_db_field"] = use_db_field
 
         return self.to_mongo(value, **ex_vars)
 
@@ -189,8 +210,8 @@ class BaseField(object):
         pass
 
     def _validate_choices(self, value):
-        Document = _import_class('Document')
-        EmbeddedDocument = _import_class('EmbeddedDocument')
+        Document = _import_class("Document")
+        EmbeddedDocument = _import_class("EmbeddedDocument")
 
         choice_list = self.choices
         if isinstance(next(iter(choice_list)), (list, tuple)):
@@ -201,13 +222,13 @@ class BaseField(object):
         if isinstance(value, (Document, EmbeddedDocument)):
             if not any(isinstance(value, c) for c in choice_list):
                 self.error(
-                    'Value must be an instance of %s' % (
-                        six.text_type(choice_list)
-                    )
+                    "Value must be an instance of %s" % (six.text_type(choice_list))
                 )
         # Choices which are types other than Documents
-        elif value not in choice_list:
-            self.error('Value must be one of %s' % six.text_type(choice_list))
+        else:
+            values = value if isinstance(value, (list, tuple)) else [value]
+            if len(set(values) - set(choice_list)):
+                self.error("Value must be one of %s" % six.text_type(choice_list))
 
     def _validate(self, value, **kwargs):
         # Check the Choices Constraint
@@ -217,11 +238,23 @@ class BaseField(object):
         # check validation argument
         if self.validation is not None:
             if callable(self.validation):
-                if not self.validation(value):
-                    self.error('Value does not match custom validation method')
+                try:
+                    # breaking change of 0.18
+                    # Get rid of True/False-type return for the validation method
+                    # in favor of having validation raising a ValidationError
+                    ret = self.validation(value)
+                    if ret is not None:
+                        raise DeprecatedError(
+                            "validation argument for `%s` must not return anything, "
+                            "it should raise a ValidationError if validation fails"
+                            % self.name
+                        )
+                except ValidationError as ex:
+                    self.error(str(ex))
             else:
-                raise ValueError('validation argument for "%s" must be a '
-                                 'callable.' % self.name)
+                raise ValueError(
+                    'validation argument for `"%s"` must be a ' "callable." % self.name
+                )
 
         self.validate(value, **kwargs)
 
@@ -255,28 +288,41 @@ class ComplexBaseField(BaseField):
             # Document class being used rather than a document object
             return self
 
-        ReferenceField = _import_class('ReferenceField')
-        GenericReferenceField = _import_class('GenericReferenceField')
-        EmbeddedDocumentListField = _import_class('EmbeddedDocumentListField')
-        dereference = (self._auto_dereference and
-                       (self.field is None or isinstance(self.field,
-                                                         (GenericReferenceField, ReferenceField))))
+        ReferenceField = _import_class("ReferenceField")
+        GenericReferenceField = _import_class("GenericReferenceField")
+        EmbeddedDocumentListField = _import_class("EmbeddedDocumentListField")
 
-        _dereference = _import_class('DeReference')()
+        auto_dereference = instance._fields[self.name]._auto_dereference
 
-        self._auto_dereference = instance._fields[self.name]._auto_dereference
-        if instance._initialised and dereference and instance._data.get(self.name):
+        dereference = auto_dereference and (
+            self.field is None
+            or isinstance(self.field, (GenericReferenceField, ReferenceField))
+        )
+
+        _dereference = _import_class("DeReference")()
+
+        if (
+            instance._initialised
+            and dereference
+            and instance._data.get(self.name)
+            and not getattr(instance._data[self.name], "_dereferenced", False)
+        ):
             instance._data[self.name] = _dereference(
-                instance._data.get(self.name), max_depth=1, instance=instance,
-                name=self.name
+                instance._data.get(self.name),
+                max_depth=1,
+                instance=instance,
+                name=self.name,
             )
+            if hasattr(instance._data[self.name], "_dereferenced"):
+                instance._data[self.name]._dereferenced = True
 
         value = super(ComplexBaseField, self).__get__(instance, owner)
 
         # Convert lists / values so we can watch for any changes on them
         if isinstance(value, (list, tuple)):
-            if (issubclass(type(self), EmbeddedDocumentListField) and
-                    not isinstance(value, EmbeddedDocumentList)):
+            if issubclass(type(self), EmbeddedDocumentListField) and not isinstance(
+                value, EmbeddedDocumentList
+            ):
                 value = EmbeddedDocumentList(value, instance, self.name)
             elif not isinstance(value, BaseList):
                 value = BaseList(value, instance, self.name)
@@ -285,12 +331,13 @@ class ComplexBaseField(BaseField):
             value = BaseDict(value, instance, self.name)
             instance._data[self.name] = value
 
-        if (self._auto_dereference and instance._initialised and
-                isinstance(value, (BaseList, BaseDict)) and
-                not value._dereferenced):
-            value = _dereference(
-                value, max_depth=1, instance=instance, name=self.name
-            )
+        if (
+            auto_dereference
+            and instance._initialised
+            and isinstance(value, (BaseList, BaseDict))
+            and not value._dereferenced
+        ):
+            value = _dereference(value, max_depth=1, instance=instance, name=self.name)
             value._dereferenced = True
             instance._data[self.name] = value
 
@@ -301,63 +348,72 @@ class ComplexBaseField(BaseField):
         if isinstance(value, six.string_types):
             return value
 
-        if hasattr(value, 'to_python'):
+        if hasattr(value, "to_python"):
             return value.to_python()
 
+        BaseDocument = _import_class("BaseDocument")
+        if isinstance(value, BaseDocument):
+            # Something is wrong, return the value as it is
+            return value
+
         is_list = False
-        if not hasattr(value, 'items'):
+        if not hasattr(value, "items"):
             try:
                 is_list = True
-                value = {k: v for k, v in enumerate(value)}
+                value = {idx: v for idx, v in enumerate(value)}
             except TypeError:  # Not iterable return the value
                 return value
 
         if self.field:
             self.field._auto_dereference = self._auto_dereference
-            value_dict = {key: self.field.to_python(item)
-                          for key, item in list(value.items())}
+            value_dict = {
+                key: self.field.to_python(item) for key, item in list(value.items())
+            }
         else:
-            Document = _import_class('Document')
+            Document = _import_class("Document")
             value_dict = {}
             for k, v in list(value.items()):
                 if isinstance(v, Document):
                     # We need the id from the saved object to create the DBRef
                     if v.pk is None:
-                        self.error('You can only reference documents once they'
-                                   ' have been saved to the database')
+                        self.error(
+                            "You can only reference documents once they"
+                            " have been saved to the database"
+                        )
                     collection = v._get_collection_name()
                     value_dict[k] = DBRef(collection, v.pk)
-                elif hasattr(v, 'to_python'):
+                elif hasattr(v, "to_python"):
                     value_dict[k] = v.to_python()
                 else:
                     value_dict[k] = self.to_python(v)
 
         if is_list:  # Convert back to a list
-            return [v for _, v in sorted(list(value_dict.items()),
-                                         key=operator.itemgetter(0))]
+            return [
+                v for _, v in sorted(list(value_dict.items()), key=operator.itemgetter(0))
+            ]
         return value_dict
 
     def to_mongo(self, value, use_db_field=True, fields=None):
         """Convert a Python type to a MongoDB-compatible type."""
-        Document = _import_class('Document')
-        EmbeddedDocument = _import_class('EmbeddedDocument')
-        GenericReferenceField = _import_class('GenericReferenceField')
+        Document = _import_class("Document")
+        EmbeddedDocument = _import_class("EmbeddedDocument")
+        GenericReferenceField = _import_class("GenericReferenceField")
 
         if isinstance(value, six.string_types):
             return value
 
-        if hasattr(value, 'to_mongo'):
+        if hasattr(value, "to_mongo"):
             if isinstance(value, Document):
                 return GenericReferenceField().to_mongo(value)
             cls = value.__class__
             val = value.to_mongo(use_db_field, fields)
             # If it's a document that is not inherited add _cls
             if isinstance(value, EmbeddedDocument):
-                val['_cls'] = cls.__name__
+                val["_cls"] = cls.__name__
             return val
 
         is_list = False
-        if not hasattr(value, 'items'):
+        if not hasattr(value, "items"):
             try:
                 is_list = True
                 value = {k: v for k, v in enumerate(value)}
@@ -367,48 +423,51 @@ class ComplexBaseField(BaseField):
         if self.field:
             value_dict = {
                 key: self.field._to_mongo_safe_call(item, use_db_field, fields)
-                for key, item in value.items()
+                for key, item in iteritems(value)
             }
         else:
             value_dict = {}
-            for k, v in value.items():
+            for k, v in iteritems(value):
                 if isinstance(v, Document):
                     # We need the id from the saved object to create the DBRef
                     if v.pk is None:
-                        self.error('You can only reference documents once they'
-                                   ' have been saved to the database')
+                        self.error(
+                            "You can only reference documents once they"
+                            " have been saved to the database"
+                        )
 
                     # If its a document that is not inheritable it won't have
                     # any _cls data so make it a generic reference allows
                     # us to dereference
-                    meta = getattr(v, '_meta', {})
-                    allow_inheritance = meta.get('allow_inheritance')
+                    meta = getattr(v, "_meta", {})
+                    allow_inheritance = meta.get("allow_inheritance")
                     if not allow_inheritance and not self.field:
                         value_dict[k] = GenericReferenceField().to_mongo(v)
                     else:
                         collection = v._get_collection_name()
                         value_dict[k] = DBRef(collection, v.pk)
-                elif hasattr(v, 'to_mongo'):
+                elif hasattr(v, "to_mongo"):
                     cls = v.__class__
                     val = v.to_mongo(use_db_field, fields)
                     # If it's a document that is not inherited add _cls
                     if isinstance(v, (Document, EmbeddedDocument)):
-                        val['_cls'] = cls.__name__
+                        val["_cls"] = cls.__name__
                     value_dict[k] = val
                 else:
                     value_dict[k] = self.to_mongo(v, use_db_field, fields)
 
         if is_list:  # Convert back to a list
-            return [v for _, v in sorted(list(value_dict.items()),
-                                         key=operator.itemgetter(0))]
+            return [
+                v for _, v in sorted(list(value_dict.items()), key=operator.itemgetter(0))
+            ]
         return value_dict
 
     def validate(self, value):
         """If field is provided ensure the value is valid."""
         errors = {}
         if self.field:
-            if hasattr(value, 'iteritems') or hasattr(value, 'items'):
-                sequence = iter(value.items())
+            if hasattr(value, "iteritems") or hasattr(value, "items"):
+                sequence = iteritems(value)
             else:
                 sequence = enumerate(value)
             for k, v in sequence:
@@ -421,11 +480,10 @@ class ComplexBaseField(BaseField):
 
             if errors:
                 field_class = self.field.__class__.__name__
-                self.error('Invalid %s item (%s)' % (field_class, value),
-                           errors=errors)
+                self.error("Invalid %s item (%s)" % (field_class, value), errors=errors)
         # Don't allow empty values if required
         if self.required and not value:
-            self.error('Field is required and cannot be empty')
+            self.error("Field is required and cannot be empty")
 
     def prepare_query_value(self, op, value):
         return self.to_mongo(value)
@@ -468,7 +526,7 @@ class ObjectIdField(BaseField):
         try:
             ObjectId(six.text_type(value))
         except Exception:
-            self.error('Invalid Object ID')
+            self.error("Invalid Object ID")
 
 
 class GeoJsonBaseField(BaseField):
@@ -478,14 +536,14 @@ class GeoJsonBaseField(BaseField):
     """
 
     _geo_index = pymongo.GEOSPHERE
-    _type = 'GeoBase'
+    _type = "GeoBase"
 
     def __init__(self, auto_index=True, *args, **kwargs):
         """
         :param bool auto_index: Automatically create a '2dsphere' index.\
             Defaults to `True`.
         """
-        self._name = '%sField' % self._type
+        self._name = "%sField" % self._type
         if not auto_index:
             self._geo_index = False
         super(GeoJsonBaseField, self).__init__(*args, **kwargs)
@@ -493,57 +551,58 @@ class GeoJsonBaseField(BaseField):
     def validate(self, value):
         """Validate the GeoJson object based on its type."""
         if isinstance(value, dict):
-            if set(value.keys()) == set(['type', 'coordinates']):
-                if value['type'] != self._type:
-                    self.error('%s type must be "%s"' %
-                               (self._name, self._type))
-                return self.validate(value['coordinates'])
+            if set(value.keys()) == {"type", "coordinates"}:
+                if value["type"] != self._type:
+                    self.error('%s type must be "%s"' % (self._name, self._type))
+                return self.validate(value["coordinates"])
             else:
-                self.error('%s can only accept a valid GeoJson dictionary'
-                           ' or lists of (x, y)' % self._name)
+                self.error(
+                    "%s can only accept a valid GeoJson dictionary"
+                    " or lists of (x, y)" % self._name
+                )
                 return
         elif not isinstance(value, (list, tuple)):
-            self.error('%s can only accept lists of [x, y]' % self._name)
+            self.error("%s can only accept lists of [x, y]" % self._name)
             return
 
-        validate = getattr(self, '_validate_%s' % self._type.lower())
+        validate = getattr(self, "_validate_%s" % self._type.lower())
         error = validate(value)
         if error:
             self.error(error)
 
     def _validate_polygon(self, value, top_level=True):
         if not isinstance(value, (list, tuple)):
-            return 'Polygons must contain list of linestrings'
+            return "Polygons must contain list of linestrings"
 
         # Quick and dirty validator
         try:
             value[0][0][0]
         except (TypeError, IndexError):
-            return 'Invalid Polygon must contain at least one valid linestring'
+            return "Invalid Polygon must contain at least one valid linestring"
 
         errors = []
         for val in value:
             error = self._validate_linestring(val, False)
             if not error and val[0] != val[-1]:
-                error = 'LineStrings must start and end at the same point'
+                error = "LineStrings must start and end at the same point"
             if error and error not in errors:
                 errors.append(error)
         if errors:
             if top_level:
-                return 'Invalid Polygon:\n%s' % ', '.join(errors)
+                return "Invalid Polygon:\n%s" % ", ".join(errors)
             else:
-                return '%s' % ', '.join(errors)
+                return "%s" % ", ".join(errors)
 
     def _validate_linestring(self, value, top_level=True):
         """Validate a linestring."""
         if not isinstance(value, (list, tuple)):
-            return 'LineStrings must contain list of coordinate pairs'
+            return "LineStrings must contain list of coordinate pairs"
 
         # Quick and dirty validator
         try:
             value[0][0]
         except (TypeError, IndexError):
-            return 'Invalid LineString must contain at least one valid point'
+            return "Invalid LineString must contain at least one valid point"
 
         errors = []
         for val in value:
@@ -552,29 +611,30 @@ class GeoJsonBaseField(BaseField):
                 errors.append(error)
         if errors:
             if top_level:
-                return 'Invalid LineString:\n%s' % ', '.join(errors)
+                return "Invalid LineString:\n%s" % ", ".join(errors)
             else:
-                return '%s' % ', '.join(errors)
+                return "%s" % ", ".join(errors)
 
     def _validate_point(self, value):
         """Validate each set of coords"""
         if not isinstance(value, (list, tuple)):
-            return 'Points must be a list of coordinate pairs'
+            return "Points must be a list of coordinate pairs"
         elif not len(value) == 2:
-            return 'Value (%s) must be a two-dimensional point' % repr(value)
-        elif (not isinstance(value[0], (float, int)) or
-              not isinstance(value[1], (float, int))):
-            return 'Both values (%s) in point must be float or int' % repr(value)
+            return "Value (%s) must be a two-dimensional point" % repr(value)
+        elif not isinstance(value[0], (float, int)) or not isinstance(
+            value[1], (float, int)
+        ):
+            return "Both values (%s) in point must be float or int" % repr(value)
 
     def _validate_multipoint(self, value):
         if not isinstance(value, (list, tuple)):
-            return 'MultiPoint must be a list of Point'
+            return "MultiPoint must be a list of Point"
 
         # Quick and dirty validator
         try:
             value[0][0]
         except (TypeError, IndexError):
-            return 'Invalid MultiPoint must contain at least one valid point'
+            return "Invalid MultiPoint must contain at least one valid point"
 
         errors = []
         for point in value:
@@ -583,17 +643,17 @@ class GeoJsonBaseField(BaseField):
                 errors.append(error)
 
         if errors:
-            return '%s' % ', '.join(errors)
+            return "%s" % ", ".join(errors)
 
     def _validate_multilinestring(self, value, top_level=True):
         if not isinstance(value, (list, tuple)):
-            return 'MultiLineString must be a list of LineString'
+            return "MultiLineString must be a list of LineString"
 
         # Quick and dirty validator
         try:
             value[0][0][0]
         except (TypeError, IndexError):
-            return 'Invalid MultiLineString must contain at least one valid linestring'
+            return "Invalid MultiLineString must contain at least one valid linestring"
 
         errors = []
         for linestring in value:
@@ -603,19 +663,19 @@ class GeoJsonBaseField(BaseField):
 
         if errors:
             if top_level:
-                return 'Invalid MultiLineString:\n%s' % ', '.join(errors)
+                return "Invalid MultiLineString:\n%s" % ", ".join(errors)
             else:
-                return '%s' % ', '.join(errors)
+                return "%s" % ", ".join(errors)
 
     def _validate_multipolygon(self, value):
         if not isinstance(value, (list, tuple)):
-            return 'MultiPolygon must be a list of Polygon'
+            return "MultiPolygon must be a list of Polygon"
 
         # Quick and dirty validator
         try:
             value[0][0][0][0]
         except (TypeError, IndexError):
-            return 'Invalid MultiPolygon must contain at least one valid Polygon'
+            return "Invalid MultiPolygon must contain at least one valid Polygon"
 
         errors = []
         for polygon in value:
@@ -624,9 +684,9 @@ class GeoJsonBaseField(BaseField):
                 errors.append(error)
 
         if errors:
-            return 'Invalid MultiPolygon:\n%s' % ', '.join(errors)
+            return "Invalid MultiPolygon:\n%s" % ", ".join(errors)
 
     def to_mongo(self, value):
         if isinstance(value, dict):
             return value
-        return SON([('type', self._type), ('coordinates', value)])
+        return SON([("type", self._type), ("coordinates", value)])
